/*******************************************************************************
* File Name: SBC.c
* Version 3.0
*
* Description:
*  This file provides the source code to the API for the SCB Component.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SBC_PVT.h"

#if (SBC_SCB_MODE_I2C_INC)
    #include "SBC_I2C_PVT.h"
#endif /* (SBC_SCB_MODE_I2C_INC) */

#if (SBC_SCB_MODE_EZI2C_INC)
    #include "SBC_EZI2C_PVT.h"
#endif /* (SBC_SCB_MODE_EZI2C_INC) */

#if (SBC_SCB_MODE_SPI_INC || SBC_SCB_MODE_UART_INC)
    #include "SBC_SPI_UART_PVT.h"
#endif /* (SBC_SCB_MODE_SPI_INC || SBC_SCB_MODE_UART_INC) */


/***************************************
*    Run Time Configuration Vars
***************************************/

/* Stores internal component configuration for Unconfigured mode */
#if (SBC_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Common configuration variables */
    uint8 SBC_scbMode = SBC_SCB_MODE_UNCONFIG;
    uint8 SBC_scbEnableWake;
    uint8 SBC_scbEnableIntr;

    /* I2C configuration variables */
    uint8 SBC_mode;
    uint8 SBC_acceptAddr;

    /* SPI/UART configuration variables */
    volatile uint8 * SBC_rxBuffer;
    uint8  SBC_rxDataBits;
    uint32 SBC_rxBufferSize;

    volatile uint8 * SBC_txBuffer;
    uint8  SBC_txDataBits;
    uint32 SBC_txBufferSize;

    /* EZI2C configuration variables */
    uint8 SBC_numberOfAddr;
    uint8 SBC_subAddrSize;
#endif /* (SBC_SCB_MODE_UNCONFIG_CONST_CFG) */


/***************************************
*     Common SCB Vars
***************************************/

uint8 SBC_initVar = 0u;

#if (SBC_SCB_IRQ_INTERNAL)
#if !defined (CY_REMOVE_SBC_CUSTOM_INTR_HANDLER)
    void (*SBC_customIntrHandler)(void) = NULL;
#endif /* !defined (CY_REMOVE_SBC_CUSTOM_INTR_HANDLER) */
#endif /* (SBC_SCB_IRQ_INTERNAL) */


/***************************************
*    Private Function Prototypes
***************************************/

static void SBC_ScbEnableIntr(void);
static void SBC_ScbModeStop(void);
static void SBC_ScbModePostEnable(void);


/*******************************************************************************
* Function Name: SBC_Init
********************************************************************************
*
* Summary:
*  Initializes the SCB component to operate in one of the selected
*  configurations: I2C, SPI, UART or EZI2C.
*  When the configuration is set to "Unconfigured SCB", this function does
*  not do any initialization. Use mode-specific initialization APIs instead:
*  SCB_I2CInit, SCB_SpiInit, SCB_UartInit or SCB_EzI2CInit.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SBC_Init(void)
{
#if (SBC_SCB_MODE_UNCONFIG_CONST_CFG)
    if (SBC_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        SBC_initVar = 0u;
    }
    else
    {
        /* Initialization was done before this function call */
    }

#elif (SBC_SCB_MODE_I2C_CONST_CFG)
    SBC_I2CInit();

#elif (SBC_SCB_MODE_SPI_CONST_CFG)
    SBC_SpiInit();

#elif (SBC_SCB_MODE_UART_CONST_CFG)
    SBC_UartInit();

#elif (SBC_SCB_MODE_EZI2C_CONST_CFG)
    SBC_EzI2CInit();

#endif /* (SBC_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SBC_Enable
********************************************************************************
*
* Summary:
*  Enables the SCB component operation.
*  The SCB configuration should be not changed when the component is enabled.
*  Any configuration changes should be made after disabling the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SBC_Enable(void)
{
#if (SBC_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Enable SCB block, only if it is already configured */
    if (!SBC_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        SBC_CTRL_REG |= SBC_CTRL_ENABLED;

        SBC_ScbEnableIntr();

        /* Call PostEnable function specific to current operation mode */
        SBC_ScbModePostEnable();
    }
#else
    SBC_CTRL_REG |= SBC_CTRL_ENABLED;

    SBC_ScbEnableIntr();

    /* Call PostEnable function specific to current operation mode */
    SBC_ScbModePostEnable();
#endif /* (SBC_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SBC_Start
********************************************************************************
*
* Summary:
*  Invokes SCB_Init() and SCB_Enable().
*  After this function call, the component is enabled and ready for operation.
*  When configuration is set to "Unconfigured SCB", the component must first be
*  initialized to operate in one of the following configurations: I2C, SPI, UART
*  or EZ I2C. Otherwise this function does not enable the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  SBC_initVar - used to check initial configuration, modified
*  on first function call.
*
*******************************************************************************/
void SBC_Start(void)
{
    if (0u == SBC_initVar)
    {
        SBC_Init();
        SBC_initVar = 1u; /* Component was initialized */
    }

    SBC_Enable();
}


/*******************************************************************************
* Function Name: SBC_Stop
********************************************************************************
*
* Summary:
*  Disables the SCB component and its interrupt.
*  It also disables all TX interrupt sources so as not to cause an unexpected
*  interrupt trigger because after the component is enabled, the TX FIFO 
*  is empty.
*
* Parameters:
*  None
*
* Return:
*  None
* 
*******************************************************************************/
void SBC_Stop(void)
{
#if (SBC_SCB_IRQ_INTERNAL)
    SBC_DisableInt();
#endif /* (SBC_SCB_IRQ_INTERNAL) */

    /* Call Stop function specific to current operation mode */
    SBC_ScbModeStop();

    /* Disable SCB IP */
    SBC_CTRL_REG &= (uint32) ~SBC_CTRL_ENABLED;

    /* Disable all TX interrupt sources so as not to cause an unexpected
    * interrupt trigger because after the component is enabled, the TX FIFO
    * is empty.
    * For SCB IP v0, it is critical as it does not mask-out interrupt
    * sources when they are disabled. This can cause a code lock-up in the
    * interrupt handler because TX FIFO cannot be loaded after the block
    * is disabled.
    */
    SBC_SetTxInterruptMode(SBC_NO_INTR_SOURCES);

#if (SBC_SCB_IRQ_INTERNAL)
    SBC_ClearPendingInt();
#endif /* (SBC_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: SBC_SetRxFifoLevel
********************************************************************************
*
* Summary:
*  Sets level in the RX FIFO to generate a RX level interrupt.
*  When the RX FIFO has more entries than the RX FIFO level an RX level
*  interrupt request is generated.
*
* Parameters:
*  level: Level in the RX FIFO to generate RX level interrupt.
*         The range of valid level values is between 0 and RX FIFO depth - 1.
*
* Return:
*  None
*
*******************************************************************************/
void SBC_SetRxFifoLevel(uint32 level)
{
    uint32 rxFifoCtrl;

    rxFifoCtrl = SBC_RX_FIFO_CTRL_REG;

    rxFifoCtrl &= ((uint32) ~SBC_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    rxFifoCtrl |= ((uint32) (SBC_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    SBC_RX_FIFO_CTRL_REG = rxFifoCtrl;
}


/*******************************************************************************
* Function Name: SBC_SetTxFifoLevel
********************************************************************************
*
* Summary:
*  Sets level in the TX FIFO to generate a TX level interrupt.
*  When the TX FIFO has more entries than the TX FIFO level an TX level
*  interrupt request is generated.
*
* Parameters:
*  level: Level in the TX FIFO to generate TX level interrupt.
*         The range of valid level values is between 0 and TX FIFO depth - 1.
*
* Return:
*  None
*
*******************************************************************************/
void SBC_SetTxFifoLevel(uint32 level)
{
    uint32 txFifoCtrl;

    txFifoCtrl = SBC_TX_FIFO_CTRL_REG;

    txFifoCtrl &= ((uint32) ~SBC_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    txFifoCtrl |= ((uint32) (SBC_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    SBC_TX_FIFO_CTRL_REG = txFifoCtrl;
}


#if (SBC_SCB_IRQ_INTERNAL)
    /*******************************************************************************
    * Function Name: SBC_SetCustomInterruptHandler
    ********************************************************************************
    *
    * Summary:
    *  Registers a function to be called by the internal interrupt handler.
    *  First the function that is registered is called, then the internal interrupt
    *  handler performs any operation such as software buffer management functions
    *  before the interrupt returns.  It is the user's responsibility not to break
    *  the software buffer operations. Only one custom handler is supported, which
    *  is the function provided by the most recent call.
    *  At the initialization time no custom handler is registered.
    *
    * Parameters:
    *  func: Pointer to the function to register.
    *        The value NULL indicates to remove the current custom interrupt
    *        handler.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SBC_SetCustomInterruptHandler(void (*func)(void))
    {
    #if !defined (CY_REMOVE_SBC_CUSTOM_INTR_HANDLER)
        SBC_customIntrHandler = func; /* Register interrupt handler */
    #else
        if (NULL != func)
        {
            /* Suppress compiler warning */
        }
    #endif /* !defined (CY_REMOVE_SBC_CUSTOM_INTR_HANDLER) */
    }
#endif /* (SBC_SCB_IRQ_INTERNAL) */


/*******************************************************************************
* Function Name: SBC_ScbModeEnableIntr
********************************************************************************
*
* Summary:
*  Enables an interrupt for a specific mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void SBC_ScbEnableIntr(void)
{
#if (SBC_SCB_IRQ_INTERNAL)
    /* Enable interrupt in NVIC */
    #if (SBC_SCB_MODE_UNCONFIG_CONST_CFG)
        if (0u != SBC_scbEnableIntr)
        {
            SBC_EnableInt();
        }

    #else
        SBC_EnableInt();

    #endif /* (SBC_SCB_MODE_UNCONFIG_CONST_CFG) */
#endif /* (SBC_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: SBC_ScbModePostEnable
********************************************************************************
*
* Summary:
*  Calls the PostEnable function for a specific operation mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void SBC_ScbModePostEnable(void)
{
#if (SBC_SCB_MODE_UNCONFIG_CONST_CFG)
#if (!SBC_CY_SCBIP_V1)
    if (SBC_SCB_MODE_SPI_RUNTM_CFG)
    {
        SBC_SpiPostEnable();
    }
    else if (SBC_SCB_MODE_UART_RUNTM_CFG)
    {
        SBC_UartPostEnable();
    }
    else
    {
        /* Unknown mode: do nothing */
    }
#endif /* (!SBC_CY_SCBIP_V1) */

#elif (SBC_SCB_MODE_SPI_CONST_CFG)
    SBC_SpiPostEnable();

#elif (SBC_SCB_MODE_UART_CONST_CFG)
    SBC_UartPostEnable();

#else
    /* Unknown mode: do nothing */
#endif /* (SBC_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SBC_ScbModeStop
********************************************************************************
*
* Summary:
*  Calls the Stop function for a specific operation mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void SBC_ScbModeStop(void)
{
#if (SBC_SCB_MODE_UNCONFIG_CONST_CFG)
    if (SBC_SCB_MODE_I2C_RUNTM_CFG)
    {
        SBC_I2CStop();
    }
    else if (SBC_SCB_MODE_EZI2C_RUNTM_CFG)
    {
        SBC_EzI2CStop();
    }
#if (!SBC_CY_SCBIP_V1)
    else if (SBC_SCB_MODE_SPI_RUNTM_CFG)
    {
        SBC_SpiStop();
    }
    else if (SBC_SCB_MODE_UART_RUNTM_CFG)
    {
        SBC_UartStop();
    }
#endif /* (!SBC_CY_SCBIP_V1) */
    else
    {
        /* Unknown mode: do nothing */
    }
#elif (SBC_SCB_MODE_I2C_CONST_CFG)
    SBC_I2CStop();

#elif (SBC_SCB_MODE_EZI2C_CONST_CFG)
    SBC_EzI2CStop();

#elif (SBC_SCB_MODE_SPI_CONST_CFG)
    SBC_SpiStop();

#elif (SBC_SCB_MODE_UART_CONST_CFG)
    SBC_UartStop();

#else
    /* Unknown mode: do nothing */
#endif /* (SBC_SCB_MODE_UNCONFIG_CONST_CFG) */
}


#if (SBC_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: SBC_SetPins
    ********************************************************************************
    *
    * Summary:
    *  Sets the pins settings accordingly to the selected operation mode.
    *  Only available in the Unconfigured operation mode. The mode specific
    *  initialization function calls it.
    *  Pins configuration is set by PSoC Creator when a specific mode of operation
    *  is selected in design time.
    *
    * Parameters:
    *  mode:      Mode of SCB operation.
    *  subMode:   Sub-mode of SCB operation. It is only required for SPI and UART
    *             modes.
    *  uartEnableMask: enables TX or RX direction and RTS and CTS signals.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SBC_SetPins(uint32 mode, uint32 subMode, uint32 uartEnableMask)
    {
        uint32 hsiomSel [SBC_SCB_PINS_NUMBER];
        uint32 pinsDm   [SBC_SCB_PINS_NUMBER];

    #if (!SBC_CY_SCBIP_V1)
        uint32 pinsInBuf = 0u;
    #endif /* (!SBC_CY_SCBIP_V1) */

        uint32 i;

        /* Set default HSIOM to GPIO and Drive Mode to Analog Hi-Z */
        for (i = 0u; i < SBC_SCB_PINS_NUMBER; i++)
        {
            hsiomSel[i]  = SBC_HSIOM_DEF_SEL;
            pinsDm[i]    = SBC_PIN_DM_ALG_HIZ;
        }

        if ((SBC_SCB_MODE_I2C   == mode) ||
           (SBC_SCB_MODE_EZI2C == mode))
        {
            hsiomSel[SBC_RX_SDA_MOSI_PIN_INDEX] = SBC_HSIOM_I2C_SEL;
            hsiomSel[SBC_TX_SCL_MISO_PIN_INDEX] = SBC_HSIOM_I2C_SEL;

            pinsDm[SBC_RX_SDA_MOSI_PIN_INDEX] = SBC_PIN_DM_OD_LO;
            pinsDm[SBC_TX_SCL_MISO_PIN_INDEX] = SBC_PIN_DM_OD_LO;
        }
    #if (!SBC_CY_SCBIP_V1)
        else if (SBC_SCB_MODE_SPI == mode)
        {
            hsiomSel[SBC_RX_SDA_MOSI_PIN_INDEX] = SBC_HSIOM_SPI_SEL;
            hsiomSel[SBC_TX_SCL_MISO_PIN_INDEX] = SBC_HSIOM_SPI_SEL;
            hsiomSel[SBC_CTS_SCLK_PIN_INDEX] = SBC_HSIOM_SPI_SEL;

            if (SBC_SPI_SLAVE == subMode)
            {
                /* Slave */
                pinsDm[SBC_RX_SDA_MOSI_PIN_INDEX] = SBC_PIN_DM_DIG_HIZ;
                pinsDm[SBC_TX_SCL_MISO_PIN_INDEX] = SBC_PIN_DM_STRONG;
                pinsDm[SBC_CTS_SCLK_PIN_INDEX] = SBC_PIN_DM_DIG_HIZ;

            #if (SBC_RTS_SS0_PIN)
                /* Only SS0 is valid choice for Slave */
                hsiomSel[SBC_RTS_SS0_PIN_INDEX] = SBC_HSIOM_SPI_SEL;
                pinsDm  [SBC_RTS_SS0_PIN_INDEX] = SBC_PIN_DM_DIG_HIZ;
            #endif /* (SBC_RTS_SS0_PIN) */

            #if (SBC_TX_SCL_MISO_PIN)
                /* Disable input buffer */
                 pinsInBuf |= SBC_TX_SCL_MISO_PIN_MASK;
            #endif /* (SBC_TX_SCL_MISO_PIN) */
            }
            else /* (Master) */
            {
                pinsDm[SBC_RX_SDA_MOSI_PIN_INDEX] = SBC_PIN_DM_STRONG;
                pinsDm[SBC_TX_SCL_MISO_PIN_INDEX] = SBC_PIN_DM_DIG_HIZ;
                pinsDm[SBC_CTS_SCLK_PIN_INDEX] = SBC_PIN_DM_STRONG;

            #if (SBC_RTS_SS0_PIN)
                hsiomSel [SBC_RTS_SS0_PIN_INDEX] = SBC_HSIOM_SPI_SEL;
                pinsDm   [SBC_RTS_SS0_PIN_INDEX] = SBC_PIN_DM_STRONG;
                pinsInBuf |= SBC_RTS_SS0_PIN_MASK;
            #endif /* (SBC_RTS_SS0_PIN) */

            #if (SBC_SS1_PIN)
                hsiomSel [SBC_SS1_PIN_INDEX] = SBC_HSIOM_SPI_SEL;
                pinsDm   [SBC_SS1_PIN_INDEX] = SBC_PIN_DM_STRONG;
                pinsInBuf |= SBC_SS1_PIN_MASK;
            #endif /* (SBC_SS1_PIN) */

            #if (SBC_SS2_PIN)
                hsiomSel [SBC_SS2_PIN_INDEX] = SBC_HSIOM_SPI_SEL;
                pinsDm   [SBC_SS2_PIN_INDEX] = SBC_PIN_DM_STRONG;
                pinsInBuf |= SBC_SS2_PIN_MASK;
            #endif /* (SBC_SS2_PIN) */

            #if (SBC_SS3_PIN)
                hsiomSel [SBC_SS3_PIN_INDEX] = SBC_HSIOM_SPI_SEL;
                pinsDm   [SBC_SS3_PIN_INDEX] = SBC_PIN_DM_STRONG;
                pinsInBuf |= SBC_SS3_PIN_MASK;
            #endif /* (SBC_SS3_PIN) */

                /* Disable input buffers */
            #if (SBC_RX_SDA_MOSI_PIN)
                pinsInBuf |= SBC_RX_SDA_MOSI_PIN_MASK;
            #endif /* (SBC_RX_SDA_MOSI_PIN) */

             #if (SBC_RX_WAKE_SDA_MOSI_PIN)
                pinsInBuf |= SBC_RX_WAKE_SDA_MOSI_PIN_MASK;
            #endif /* (SBC_RX_WAKE_SDA_MOSI_PIN) */

            #if (SBC_CTS_SCLK_PIN)
                pinsInBuf |= SBC_CTS_SCLK_PIN_MASK;
            #endif /* (SBC_CTS_SCLK_PIN) */
            }
        }
        else /* UART */
        {
            if (SBC_UART_MODE_SMARTCARD == subMode)
            {
                /* SmartCard */
                hsiomSel[SBC_TX_SCL_MISO_PIN_INDEX] = SBC_HSIOM_UART_SEL;
                pinsDm  [SBC_TX_SCL_MISO_PIN_INDEX] = SBC_PIN_DM_OD_LO;
            }
            else /* Standard or IrDA */
            {
                if (0u != (SBC_UART_RX_PIN_ENABLE & uartEnableMask))
                {
                    hsiomSel[SBC_RX_SDA_MOSI_PIN_INDEX] = SBC_HSIOM_UART_SEL;
                    pinsDm  [SBC_RX_SDA_MOSI_PIN_INDEX] = SBC_PIN_DM_DIG_HIZ;
                }

                if (0u != (SBC_UART_TX_PIN_ENABLE & uartEnableMask))
                {
                    hsiomSel[SBC_TX_SCL_MISO_PIN_INDEX] = SBC_HSIOM_UART_SEL;
                    pinsDm  [SBC_TX_SCL_MISO_PIN_INDEX] = SBC_PIN_DM_STRONG;

                #if (SBC_TX_SCL_MISO_PIN)
                     pinsInBuf |= SBC_TX_SCL_MISO_PIN_MASK;
                #endif /* (SBC_TX_SCL_MISO_PIN) */
                }

            #if !(SBC_CY_SCBIP_V0 || SBC_CY_SCBIP_V1)
                if (SBC_UART_MODE_STD == subMode)
                {
                    if (0u != (SBC_UART_CTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* CTS input is multiplexed with SCLK */
                        hsiomSel[SBC_CTS_SCLK_PIN_INDEX] = SBC_HSIOM_UART_SEL;
                        pinsDm  [SBC_CTS_SCLK_PIN_INDEX] = SBC_PIN_DM_DIG_HIZ;
                    }

                    if (0u != (SBC_UART_RTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* RTS output is multiplexed with SS0 */
                        hsiomSel[SBC_RTS_SS0_PIN_INDEX] = SBC_HSIOM_UART_SEL;
                        pinsDm  [SBC_RTS_SS0_PIN_INDEX] = SBC_PIN_DM_STRONG;

                    #if (SBC_RTS_SS0_PIN)
                        /* Disable input buffer */
                        pinsInBuf |= SBC_RTS_SS0_PIN_MASK;
                    #endif /* (SBC_RTS_SS0_PIN) */
                    }
                }
            #endif /* !(SBC_CY_SCBIP_V0 || SBC_CY_SCBIP_V1) */
            }
        }
    #endif /* (!SBC_CY_SCBIP_V1) */

    /* Configure pins: set HSIOM, DM and InputBufEnable */
    /* Note: the DR register settings do not effect the pin output if HSIOM is other than GPIO */

    #if (SBC_RX_WAKE_SDA_MOSI_PIN)
        SBC_SET_HSIOM_SEL(SBC_RX_WAKE_SDA_MOSI_HSIOM_REG,
                                       SBC_RX_WAKE_SDA_MOSI_HSIOM_MASK,
                                       SBC_RX_WAKE_SDA_MOSI_HSIOM_POS,
                                       hsiomSel[SBC_RX_WAKE_SDA_MOSI_PIN_INDEX]);

        SBC_uart_rx_wake_i2c_sda_spi_mosi_SetDriveMode((uint8)
                                                               pinsDm[SBC_RX_WAKE_SDA_MOSI_PIN_INDEX]);

        SBC_SET_INP_DIS(SBC_uart_rx_wake_i2c_sda_spi_mosi_INP_DIS,
                                     SBC_uart_rx_wake_i2c_sda_spi_mosi_MASK,
                                     (0u != (pinsInBuf & SBC_RX_WAKE_SDA_MOSI_PIN_MASK)));

         /* Set interrupt on falling edge */
        SBC_SET_INCFG_TYPE(SBC_RX_WAKE_SDA_MOSI_INTCFG_REG,
                                        SBC_RX_WAKE_SDA_MOSI_INTCFG_TYPE_MASK,
                                        SBC_RX_WAKE_SDA_MOSI_INTCFG_TYPE_POS,
                                        SBC_INTCFG_TYPE_FALLING_EDGE);
    #endif /* (SBC_RX_WAKE_SDA_MOSI_PIN) */

    #if (SBC_RX_SDA_MOSI_PIN)
        SBC_SET_HSIOM_SEL(SBC_RX_SDA_MOSI_HSIOM_REG,
                                       SBC_RX_SDA_MOSI_HSIOM_MASK,
                                       SBC_RX_SDA_MOSI_HSIOM_POS,
                                        hsiomSel[SBC_RX_SDA_MOSI_PIN_INDEX]);

        SBC_uart_rx_i2c_sda_spi_mosi_SetDriveMode((uint8) pinsDm[SBC_RX_SDA_MOSI_PIN_INDEX]);

    #if (!SBC_CY_SCBIP_V1)
        SBC_SET_INP_DIS(SBC_uart_rx_i2c_sda_spi_mosi_INP_DIS,
                                     SBC_uart_rx_i2c_sda_spi_mosi_MASK,
                                     (0u != (pinsInBuf & SBC_RX_SDA_MOSI_PIN_MASK)));
    #endif /* (!SBC_CY_SCBIP_V1) */
    #endif /* (SBC_RX_SDA_MOSI_PIN) */

    #if (SBC_TX_SCL_MISO_PIN)
        SBC_SET_HSIOM_SEL(SBC_TX_SCL_MISO_HSIOM_REG,
                                       SBC_TX_SCL_MISO_HSIOM_MASK,
                                       SBC_TX_SCL_MISO_HSIOM_POS,
                                        hsiomSel[SBC_TX_SCL_MISO_PIN_INDEX]);

        SBC_uart_tx_i2c_scl_spi_miso_SetDriveMode((uint8) pinsDm[SBC_TX_SCL_MISO_PIN_INDEX]);

    #if (!SBC_CY_SCBIP_V1)
        SBC_SET_INP_DIS(SBC_uart_tx_i2c_scl_spi_miso_INP_DIS,
                                     SBC_uart_tx_i2c_scl_spi_miso_MASK,
                                    (0u != (pinsInBuf & SBC_TX_SCL_MISO_PIN_MASK)));
    #endif /* (!SBC_CY_SCBIP_V1) */
    #endif /* (SBC_RX_SDA_MOSI_PIN) */

    #if (SBC_CTS_SCLK_PIN)
        SBC_SET_HSIOM_SEL(SBC_CTS_SCLK_HSIOM_REG,
                                       SBC_CTS_SCLK_HSIOM_MASK,
                                       SBC_CTS_SCLK_HSIOM_POS,
                                       hsiomSel[SBC_CTS_SCLK_PIN_INDEX]);

        SBC_uart_cts_spi_sclk_SetDriveMode((uint8) pinsDm[SBC_CTS_SCLK_PIN_INDEX]);

        SBC_SET_INP_DIS(SBC_uart_cts_spi_sclk_INP_DIS,
                                     SBC_uart_cts_spi_sclk_MASK,
                                     (0u != (pinsInBuf & SBC_CTS_SCLK_PIN_MASK)));
    #endif /* (SBC_CTS_SCLK_PIN) */

    #if (SBC_RTS_SS0_PIN)
        SBC_SET_HSIOM_SEL(SBC_RTS_SS0_HSIOM_REG,
                                       SBC_RTS_SS0_HSIOM_MASK,
                                       SBC_RTS_SS0_HSIOM_POS,
                                       hsiomSel[SBC_RTS_SS0_PIN_INDEX]);

        SBC_uart_rts_spi_ss0_SetDriveMode((uint8) pinsDm[SBC_RTS_SS0_PIN_INDEX]);

        SBC_SET_INP_DIS(SBC_uart_rts_spi_ss0_INP_DIS,
                                     SBC_uart_rts_spi_ss0_MASK,
                                     (0u != (pinsInBuf & SBC_RTS_SS0_PIN_MASK)));
    #endif /* (SBC_RTS_SS0_PIN) */

    #if (SBC_SS1_PIN)
        SBC_SET_HSIOM_SEL(SBC_SS1_HSIOM_REG,
                                       SBC_SS1_HSIOM_MASK,
                                       SBC_SS1_HSIOM_POS,
                                       hsiomSel[SBC_SS1_PIN_INDEX]);

        SBC_spi_ss1_SetDriveMode((uint8) pinsDm[SBC_SS1_PIN_INDEX]);

        SBC_SET_INP_DIS(SBC_spi_ss1_INP_DIS,
                                     SBC_spi_ss1_MASK,
                                     (0u != (pinsInBuf & SBC_SS1_PIN_MASK)));
    #endif /* (SBC_SS1_PIN) */

    #if (SBC_SS2_PIN)
        SBC_SET_HSIOM_SEL(SBC_SS2_HSIOM_REG,
                                       SBC_SS2_HSIOM_MASK,
                                       SBC_SS2_HSIOM_POS,
                                       hsiomSel[SBC_SS2_PIN_INDEX]);

        SBC_spi_ss2_SetDriveMode((uint8) pinsDm[SBC_SS2_PIN_INDEX]);

        SBC_SET_INP_DIS(SBC_spi_ss2_INP_DIS,
                                     SBC_spi_ss2_MASK,
                                     (0u != (pinsInBuf & SBC_SS2_PIN_MASK)));
    #endif /* (SBC_SS2_PIN) */

    #if (SBC_SS3_PIN)
        SBC_SET_HSIOM_SEL(SBC_SS3_HSIOM_REG,
                                       SBC_SS3_HSIOM_MASK,
                                       SBC_SS3_HSIOM_POS,
                                       hsiomSel[SBC_SS3_PIN_INDEX]);

        SBC_spi_ss3_SetDriveMode((uint8) pinsDm[SBC_SS3_PIN_INDEX]);

        SBC_SET_INP_DIS(SBC_spi_ss3_INP_DIS,
                                     SBC_spi_ss3_MASK,
                                     (0u != (pinsInBuf & SBC_SS3_PIN_MASK)));
    #endif /* (SBC_SS3_PIN) */
    }

#endif /* (SBC_SCB_MODE_UNCONFIG_CONST_CFG) */


#if (SBC_CY_SCBIP_V0 || SBC_CY_SCBIP_V1)
    /*******************************************************************************
    * Function Name: SBC_I2CSlaveNackGeneration
    ********************************************************************************
    *
    * Summary:
    *  Sets command to generate NACK to the address or data.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SBC_I2CSlaveNackGeneration(void)
    {
        /* Check for EC_AM toggle condition: EC_AM and clock stretching for address are enabled */
        if ((0u != (SBC_CTRL_REG & SBC_CTRL_EC_AM_MODE)) &&
            (0u == (SBC_I2C_CTRL_REG & SBC_I2C_CTRL_S_NOT_READY_ADDR_NACK)))
        {
            /* Toggle EC_AM before NACK generation */
            SBC_CTRL_REG &= ~SBC_CTRL_EC_AM_MODE;
            SBC_CTRL_REG |=  SBC_CTRL_EC_AM_MODE;
        }

        SBC_I2C_SLAVE_CMD_REG = SBC_I2C_SLAVE_CMD_S_NACK;
    }
#endif /* (SBC_CY_SCBIP_V0 || SBC_CY_SCBIP_V1) */


/* [] END OF FILE */
