/*******************************************************************************
* File Name: SBC_EZI2C.c
* Version 3.0
*
* Description:
*  This file provides the source code to the API for the SCB Component in
*  EZI2C mode.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SBC_PVT.h"
#include "SBC_EZI2C_PVT.h"


/***************************************
*      EZI2C Private Vars
***************************************/

volatile uint8 SBC_curStatus; /* Status byte */
uint8 SBC_fsmState;           /* FSM state   */

/* Variables intended to be used with Buffer 1: Primary slave address */
volatile uint8 * SBC_dataBuffer1; /* Pointer to data buffer 1 */
uint16 SBC_bufSizeBuf1; /* Size of buffer 1 in bytes      */
uint16 SBC_protectBuf1; /* Start index of write protected area buffer 1 */
uint16 SBC_offsetBuf1; /* Current offset within buffer 1 */
uint16 SBC_indexBuf1;  /* Current index within buffer 1  */

#if(SBC_SECONDARY_ADDRESS_ENABLE_CONST)
    uint8 SBC_addrBuf1; /* Primary slave address. Used for software comparison   */
    uint8 SBC_addrBuf2; /* Secondary slave address. Used for software comparison */

    /* Variables intended to be used with Buffer 2: Primary slave address */
    volatile uint8 * SBC_dataBuffer2; /* Pointer to data buffer 2 */
    uint16 SBC_bufSizeBuf2; /* Size of buffer 2 in bytes      */
    uint16 SBC_protectBuf2; /* Start index of write protected area buffer 2 */
    uint16 SBC_offsetBuf2; /* Current offset within buffer 2 */
    uint16 SBC_indexBuf2;  /* Current index within buffer 2  */
#endif /* (SBC_SECONDARY_ADDRESS_ENABLE_CONST) */


/***************************************
*      EZI2C Private Functions
***************************************/

#if(SBC_SECONDARY_ADDRESS_ENABLE_CONST)
    static uint32 SBC_EzI2CUpdateRxMatchReg(uint32 addr1, uint32 addr2);
#endif /* (SBC_SECONDARY_ADDRESS_ENABLE_CONST) */

#if(SBC_SCB_MODE_UNCONFIG_CONST_CFG)

    /***************************************
    *  Configuration Structure Initialization
    ***************************************/

    /* Constant configuration of EZI2C */
    const SBC_EZI2C_INIT_STRUCT SBC_configEzI2C =
    {
        SBC_EZI2C_CLOCK_STRETCHING,
        SBC_EZI2C_MEDIAN_FILTER_ENABLE,
        SBC_EZI2C_NUMBER_OF_ADDRESSES,
        SBC_EZI2C_PRIMARY_SLAVE_ADDRESS,
        SBC_EZI2C_SECONDARY_SLAVE_ADDRESS,
        SBC_EZI2C_SUB_ADDRESS_SIZE,
        SBC_EZI2C_WAKE_ENABLE,
        SBC_EZI2C_BYTE_MODE_ENABLE
    };

    /*******************************************************************************
    * Function Name: SBC_EzI2CInit
    ********************************************************************************
    *
    * Summary:
    *  Configures the SCB for the EZI2C operation.
    *
    * Parameters:
    *  config:  Pointer to a structure that contains the following ordered list of
    *           fields. These fields match the selections available in the
    *           customizer.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SBC_EzI2CInit(const SBC_EZI2C_INIT_STRUCT *config)
    {
        if(NULL == config)
        {
            CYASSERT(0u != 0u); /* Halt execution due to bad function parameter */
        }
        else
        {
            /* Configure pins */
            SBC_SetPins(SBC_SCB_MODE_EZI2C, SBC_DUMMY_PARAM,
                                                                      SBC_DUMMY_PARAM);

            /* Store internal configuration */
            SBC_scbMode       = (uint8) SBC_SCB_MODE_EZI2C;
            SBC_scbEnableWake = (uint8) config->enableWake;
            SBC_scbEnableIntr = (uint8) SBC_SCB_IRQ_INTERNAL;

            SBC_numberOfAddr  = (uint8) config->numberOfAddresses;
            SBC_subAddrSize   = (uint8) config->subAddrSize;

        #if (SBC_CY_SCBIP_V0)
            /* Adjust SDA filter settings. Ticket ID#150521 */
            SBC_SET_I2C_CFG_SDA_FILT_TRIM(SBC_EC_AM_I2C_CFG_SDA_FILT_TRIM);
        #endif /* (SBC_CY_SCBIP_V0) */

            /* Adjust AF and DF filter settings, AF = 1, DF = 0. Ticket ID#176179 */
            SBC_I2C_CFG_ANALOG_FITER_ENABLE;

            /* Configure I2C interface */
            SBC_CTRL_REG     = SBC_GET_CTRL_BYTE_MODE  (config->enableByteMode)    |
                                            SBC_GET_CTRL_ADDR_ACCEPT(config->numberOfAddresses) |
                                            SBC_GET_CTRL_EC_AM_MODE (config->enableWake);

            SBC_I2C_CTRL_REG = SBC_EZI2C_CTRL;

            /* Configure RX direction */
            SBC_RX_CTRL_REG = SBC_EZI2C_RX_CTRL |
                                        SBC_GET_RX_CTRL_MEDIAN(SBC_DIGITAL_FILTER_DISABLE);
                                                ;
            SBC_RX_FIFO_CTRL_REG = SBC_CLEAR_REG;

            /* Set default address and mask */
            if(SBC_EZI2C_PRIMARY_ADDRESS == config->numberOfAddresses)
            {
                SBC_RX_MATCH_REG = SBC_EzI2CUpdateRxMatchReg(config->primarySlaveAddr,
                                                                                       config->primarySlaveAddr);
            }
            else
            {
                SBC_RX_MATCH_REG = SBC_EzI2CUpdateRxMatchReg(config->primarySlaveAddr,
                                                                                       config->secondarySlaveAddr);
            }

            /* Configure TX direction */
            SBC_TX_CTRL_REG      = SBC_EZI2C_TX_CTRL;
            SBC_TX_FIFO_CTRL_REG = ((0u != (config->enableClockStretch)) ?
                                                 SBC_CLEAR_REG : SBC_EZI2C_TX_FIFO_CTRL);

            /* Configure interrupt sources */
        #if (!SBC_CY_SCBIP_V1)
           SBC_INTR_SPI_EC_MASK_REG = SBC_NO_INTR_SOURCES;
        #endif /* (!SBC_CY_SCBIP_V1) */

            SBC_INTR_I2C_EC_MASK_REG = SBC_NO_INTR_SOURCES;
            SBC_INTR_MASTER_MASK_REG = SBC_NO_INTR_SOURCES;
            SBC_INTR_SLAVE_MASK_REG  = SBC_EZI2C_INTR_SLAVE_MASK;
            SBC_INTR_TX_MASK_REG     = SBC_NO_INTR_SOURCES;

            /* Configure interrupt with EZI2C handler but do not enable it */
            CyIntDisable    (SBC_ISR_NUMBER);
            CyIntSetPriority(SBC_ISR_NUMBER, SBC_ISR_PRIORITY);
            (void) CyIntSetVector(SBC_ISR_NUMBER, (0u != (config->enableClockStretch)) ?
                                                                      (&SBC_EZI2C_STRETCH_ISR) :
                                                                      (&SBC_EZI2C_NO_STRETCH_ISR));

            if(0u != (config->enableClockStretch))
            {
                /* Configure interrupt sources */
                SBC_INTR_SLAVE_MASK_REG |= SBC_INTR_SLAVE_I2C_ADDR_MATCH;
                SBC_INTR_RX_MASK_REG     = SBC_NO_INTR_SOURCES;
            }
            else
            {
                /* Enable Auto ACK/NACK features */
                SBC_I2C_CTRL_REG |= SBC_EZI2C_CTRL_AUTO;

                /* Configure RX interrupt source */
                SBC_INTR_SLAVE_MASK_REG |= SBC_INTR_SLAVE_I2C_START;
                SBC_INTR_RX_MASK_REG     = SBC_INTR_RX_NOT_EMPTY;
            }

            /* Configure global variables */
            SBC_fsmState = SBC_EZI2C_FSM_IDLE;

            SBC_curStatus  = 0u;
            SBC_indexBuf1  = 0u;
            SBC_offsetBuf1 = 0u;
            SBC_indexBuf2  = 0u;
            SBC_offsetBuf2 = 0u;

            SBC_addrBuf1 = (uint8) config->primarySlaveAddr;
            SBC_addrBuf2 = (uint8) config->secondarySlaveAddr;
        }
    }

#else

    /*******************************************************************************
    * Function Name: SBC_EzI2CInit
    ********************************************************************************
    *
    * Summary:
    *  Configures the SCB for the EZI2C operation.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SBC_EzI2CInit(void)
    {
        /* Configure I2C interface */
        SBC_CTRL_REG     = SBC_EZI2C_DEFAULT_CTRL;
        SBC_I2C_CTRL_REG = SBC_EZI2C_DEFAULT_I2C_CTRL;

    #if (SBC_CY_SCBIP_V0)
        /* Adjust SDA filter settings. Ticket ID#150521 */
        SBC_SET_I2C_CFG_SDA_FILT_TRIM(SBC_EC_AM_I2C_CFG_SDA_FILT_TRIM);
    #endif /* (SBC_CY_SCBIP_V0) */

        /* Configure RX direction */
        SBC_RX_CTRL_REG      = SBC_EZI2C_DEFAULT_RX_CTRL;
        SBC_RX_FIFO_CTRL_REG = SBC_EZI2C_DEFAULT_RX_FIFO_CTRL;

        /* Set default address and mask */
        SBC_RX_MATCH_REG     = SBC_EZI2C_DEFAULT_RX_MATCH;

        /* Configure TX direction */
        SBC_TX_CTRL_REG      = SBC_EZI2C_DEFAULT_TX_CTRL;
        SBC_TX_FIFO_CTRL_REG = SBC_EZI2C_DEFAULT_TX_FIFO_CTRL;

        /* Configure interrupt with EZI2C handler but do not enable it */
    #if !defined (CY_EXTERNAL_INTERRUPT_CONFIG)
        CyIntDisable    (SBC_ISR_NUMBER);
        CyIntSetPriority(SBC_ISR_NUMBER, SBC_ISR_PRIORITY);

    #if (SBC_EZI2C_SCL_STRETCH_ENABLE_CONST)
        (void) CyIntSetVector(SBC_ISR_NUMBER, &SBC_EZI2C_STRETCH_ISR);
    #else
        (void) CyIntSetVector(SBC_ISR_NUMBER, &SBC_EZI2C_NO_STRETCH_ISR);
    #endif /* (SBC_EZI2C_SCL_STRETCH_ENABLE_CONST) */

    #endif /* !defined (CY_EXTERNAL_INTERRUPT_CONFIG) */

        /* Configure interrupt sources */
    #if (!SBC_CY_SCBIP_V1)
        SBC_INTR_SPI_EC_MASK_REG = SBC_EZI2C_DEFAULT_INTR_SPI_EC_MASK;
    #endif /* (!SBC_CY_SCBIP_V1) */

        SBC_INTR_I2C_EC_MASK_REG = SBC_EZI2C_DEFAULT_INTR_I2C_EC_MASK;
        SBC_INTR_SLAVE_MASK_REG  = SBC_EZI2C_DEFAULT_INTR_SLAVE_MASK;
        SBC_INTR_MASTER_MASK_REG = SBC_EZI2C_DEFAULT_INTR_MASTER_MASK;
        SBC_INTR_RX_MASK_REG     = SBC_EZI2C_DEFAULT_INTR_RX_MASK;
        SBC_INTR_TX_MASK_REG     = SBC_EZI2C_DEFAULT_INTR_TX_MASK;

        /* Configure global variables */
        SBC_fsmState = SBC_EZI2C_FSM_IDLE;

        SBC_curStatus  = 0u;
        SBC_indexBuf1  = 0u;
        SBC_offsetBuf1 = 0u;

    #if(SBC_SECONDARY_ADDRESS_ENABLE_CONST)
        SBC_indexBuf2  = 0u;
        SBC_offsetBuf2 = 0u;

        SBC_addrBuf1 = SBC_EZI2C_PRIMARY_SLAVE_ADDRESS;
        SBC_addrBuf2 = SBC_EZI2C_SECONDARY_SLAVE_ADDRESS;
    #endif /* (SBC_SECONDARY_ADDRESS_ENABLE_CONST) */
    }
#endif /* (SBC_SCB_MODE_UNCONFIG_CONST_CFG) */


/*******************************************************************************
* Function Name: SBC_EzI2CStop
********************************************************************************
*
* Summary:
*  Resets the EZI2C FSM into the default state.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SBC_EzI2CStop(void)
{
    SBC_fsmState = SBC_EZI2C_FSM_IDLE;
}


/*******************************************************************************
* Function Name: SBC_EzI2CGetActivity
********************************************************************************
*
* Summary:
*  Returns the EZI2C slave status.
*  The read, write and error status flags reset to zero after this function
*  call. The busy status flag is cleared when the transaction intended for
*  the EZI2C slave completes.
*
* Parameters:
*  None
*
* Return:
*  Returns the status of the EZI2C Slave activity.
*
*******************************************************************************/
uint32 SBC_EzI2CGetActivity(void)
{
    uint32 status;

    SBC_DisableInt();  /* Lock from interruption */

    status = SBC_curStatus;

    /* Relay on address match event from HW as bus busy status */
    #if(SBC_EZI2C_SCL_STRETCH_DISABLE)
    {
        /* For SBC_CY_SCBIP_V0 the wake is prohibited by customizer */
        #if(SBC_EZI2C_EC_AM_ENABLE)
        {
            status |= SBC_CHECK_INTR_I2C_EC(SBC_INTR_I2C_EC_WAKE_UP) ?
                        SBC_EZI2C_STATUS_BUSY : 0u;
        }
        #else
        {
            status |= SBC_CHECK_INTR_SLAVE(SBC_INTR_SLAVE_I2C_ADDR_MATCH) ?
                        SBC_EZI2C_STATUS_BUSY : 0u;
        }
        #endif
    }
    #endif

    SBC_curStatus &= ((uint8) ~SBC_EZI2C_CLEAR_STATUS);

    SBC_EnableInt();   /* Release lock */

    return(status);
}


/*******************************************************************************
* Function Name: SBC_EzI2CSetAddress1
********************************************************************************
*
* Summary:
*  Sets the primary I2C slave address.
*
* Parameters:
*  address: I2C slave address for the secondary device.
*           This address is the 7-bit right-justified slave address and does
*           not include the R/W bit.
*           This value may be any address between 0 and 127 (0x00 to 0x7F).
*
* Return:
*  None
*
*******************************************************************************/
void SBC_EzI2CSetAddress1(uint32 address)
{
    #if(SBC_SECONDARY_ADDRESS_ENABLE)
    {
        SBC_addrBuf1 = (uint8) address;

        SBC_RX_MATCH_REG = SBC_EzI2CUpdateRxMatchReg(address,
                                                                               (uint32) SBC_addrBuf2);
    }
    #else
    {
        uint32 matchReg;

        matchReg = SBC_RX_MATCH_REG;

        matchReg &= ((uint32) ~SBC_RX_MATCH_ADDR_MASK);            /* Clear address bits */
        matchReg |= ((uint32)  SBC_GET_I2C_8BIT_ADDRESS(address)); /* Set mask */

        SBC_RX_MATCH_REG = matchReg;
    }
    #endif
}


/*******************************************************************************
* Function Name: SBC_EzI2CGetAddress1
********************************************************************************
*
* Summary:
*  Gets the primary I2C slave 7-bit address.
*
* Parameters:
*  None
*
* Return:
*  Returns I2C slave address for the primary device.
*  This address is the 7-bit right-justified slave address and does not
*  include the R/W bit.
*  This value may be any address between 0 and 127 (0x00 to 0x7F).
*
*******************************************************************************/
uint32 SBC_EzI2CGetAddress1(void)
{
    uint32 address;

    #if(SBC_SECONDARY_ADDRESS_ENABLE)
    {
        address = (uint32) SBC_addrBuf1;
    }
    #else
    {
        address = (SBC_GET_RX_MATCH_ADDR(SBC_RX_MATCH_REG) >>
                   SBC_I2C_SLAVE_ADDR_POS);
    }
    #endif

    return(address);
}


/*******************************************************************************
* Function Name: SBC_EzI2CSetBuffer1
********************************************************************************
*
* Summary:
*  Sets up the data buffer to be exposed to the I2C master on a primary slave
*  address request.
*
* Parameters:
*  bufSize: Size of the buffer in bytes.
*  rwBoundary: Sets how many bytes are writeable in the beginning of the buffer.
*              This value must be less than or equal to the buffer size.
*              Data located at offset rwBoundry and greater are read only.
*  buffer: Pointer to the data buffer.
*
* Return:
*  None
*
*******************************************************************************/
void SBC_EzI2CSetBuffer1(uint32 bufSize, uint32 rwBoundary, volatile uint8 * buffer)
{
    if (NULL != buffer)
    {
        SBC_DisableInt();  /* Lock from interruption */

        SBC_dataBuffer1 =  buffer;
        SBC_bufSizeBuf1 = (uint16) bufSize;
        SBC_protectBuf1 = (uint16) rwBoundary;

        SBC_EnableInt();   /* Release lock */
    }
}


/*******************************************************************************
* Function Name: SBC_EzI2CSetReadBoundaryBuffer1
********************************************************************************
*
* Summary:
*  Sets the read only boundary in the data buffer to be exposed to the I2C
*  master on a primary slave address request.
*
* Parameters:
*  rwBoundry: Sets how many bytes are writeable in the beginning of the buffer.
*             This value must be less than or equal to the buffer size.
*             Data located at offset rwBoundry and greater are read only.
*
* Return:
*  None
*
*******************************************************************************/
void SBC_EzI2CSetReadBoundaryBuffer1(uint32 rwBoundary)
{
    SBC_protectBuf1 = (uint16) rwBoundary;
}


#if(SBC_SECONDARY_ADDRESS_ENABLE_CONST)
    /*******************************************************************************
    * Function Name: SBC_EzI2CUpdateRxMatchReg
    ********************************************************************************
    *
    * Summary:
    *  Returns the value of the RX MATCH register for addr1 and addr2. The addr1 is
    *  accepted as the primary address and it is written to RX_MATCH.ADDRESS
    *  (addr1 << 0x01).
    *  The RX_MATCH.MASK is set as follow: addr1 and addr2 equal bits set to 1
    *  otherwise 0.
    *
    * Parameters:
    *  addr1: I2C slave address for the primary device.
    *  addr2: I2C slave address for the secondary device.
    *         This address is the 7-bit right-justified slave address and does
    *         not include the R/W bit.
    *         This value may be any address between 0 and 127 (0x00 to 0x7F).
    *
    * Return:
    *  Value of RX MATCH register.
    *
    *******************************************************************************/
    static uint32 SBC_EzI2CUpdateRxMatchReg(uint32 addr1, uint32 addr2)
    {
        uint32 matchReg;

        matchReg  = ~(addr1 ^ addr2); /* If (addr1 and addr2) bit matches - mask bit equals 1, in other case 0 */

        matchReg  = (uint32) (SBC_GET_I2C_8BIT_ADDRESS(matchReg) << SBC_RX_MATCH_MASK_POS);
        matchReg |= SBC_GET_I2C_8BIT_ADDRESS(addr1);

        return(matchReg);
    }

    /*******************************************************************************
    * Function Name: SBC_EzI2CSetAddress2
    ********************************************************************************
    *
    * Summary:
    *  Sets the secondary I2C slave address.
    *
    * Parameters:
    *  address: I2C slave address for the secondary device.
    *           This address is the 7-bit right-justified slave address and does
    *           not include the R/W bit.
    *           This value may be any address between 0 and 127 (0x00 to 0x7F).
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SBC_EzI2CSetAddress2(uint32 address)
    {
        SBC_addrBuf2 = (uint8) address;

        SBC_RX_MATCH_REG = SBC_EzI2CUpdateRxMatchReg((uint32) SBC_addrBuf1,
                                                                               address);
    }


    /*******************************************************************************
    * Function Name: SBC_EzI2CGetAddress2
    ********************************************************************************
    *
    * Summary:
    *  Gets secondary the I2C slave 7-bit address.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Returns the I2C slave address for the secondary device.
    *  This address is a 7-bit right-justified slave address and does not
    *  include the R/W bit.
    *  This value may be any address between 0 and 127 (0x00 to 0x7F).
    *
    *******************************************************************************/
    uint32 SBC_EzI2CGetAddress2(void)
    {
        return((uint32) SBC_addrBuf2);
    }


    /*******************************************************************************
    * Function Name: SBC_EzI2CSetBuffer2
    ********************************************************************************
    *
    * Summary:
    *  Sets up the data buffer to be exposed to the I2C master on a secondary slave
    *  address request.
    *
    * Parameters:
    *  bufSize: Size of the buffer in bytes.
    *  rwBoundary: Sets how many bytes are writeable in the beginning of the buffer.
    *              This value must be less than or equal to the buffer size.
    *              Data located at offset rwBoundry and greater are read only.
    *  buffer: Pointer to the data buffer.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SBC_EzI2CSetBuffer2(uint32 bufSize, uint32 rwBoundary, volatile uint8 * buffer)
    {
        if (NULL != buffer)
        {
            SBC_DisableInt();  /* Lock from interruption */

            SBC_dataBuffer2 =  buffer;
            SBC_bufSizeBuf2 = (uint16) bufSize;
            SBC_protectBuf2 = (uint16) rwBoundary;

            SBC_EnableInt();   /* Release lock */
        }
    }


    /*******************************************************************************
    * Function Name: SBC_EzI2CSetReadBoundaryBuffer2
    ********************************************************************************
    *
    * Summary:
    *  Sets the read only boundary in the data buffer to be exposed to the I2C
    *  master on a secondary address request.
    *
    * Parameters:
    *  rwBoundary: Sets how many bytes are writeable at the beginning of the buffer.
    *              This value must be less than or equal to the buffer size.
    *              Data located at offset rwBoundry and greater are read only.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SBC_EzI2CSetReadBoundaryBuffer2(uint32 rwBoundary)
    {
        SBC_protectBuf2 = (uint16) rwBoundary;
    }

#endif /* (SBC_SECONDARY_ADDRESS_ENABLE_CONST) */


#if(SBC_EZI2C_WAKE_ENABLE_CONST)
    /*******************************************************************************
    * Function Name: SBC_EzI2CSaveConfig
    ********************************************************************************
    *
    * Summary:
    *  Clock stretching is  enabled: Enables INTR_I2C_EC.WAKE_UP interrupt source.
    *  It triggers on the slave address match.
    *  Clock stretching is disabled: Waits until the I2C slave becomes free and
    *  disables the block to perform reconfiguration from the active mode operation
    *  to deep sleep with wake up on the address match: enables INTR_I2C_EC.WAKE_UP
    *  interrupt source and disables the INTR_S and INTR_TX interrupt sources.
    *  The block is disabled before reconfiguration and enabled when
    *  it is completed.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SBC_EzI2CSaveConfig(void)
    {
    #if(SBC_CY_SCBIP_V0)

        #if(SBC_EZI2C_SCL_STRETCH_ENABLE)
        {
            /* Enable wakeup interrupt source on address match */
            SBC_SetI2CExtClkInterruptMode(SBC_INTR_I2C_EC_WAKE_UP);
        }
        #endif

    #else
        uint8 enableInt;

        enableInt = (uint8) SBC_INTR_I2C_EC_WAKE_UP;

        #if(SBC_EZI2C_SCL_STRETCH_ENABLE)
        {
        #if (SBC_SCB_CLK_INTERNAL)
            /* Disable clock to internal address match logic. Ticket ID #187931 */
            SBC_SCBCLK_Stop();
        #endif /* (SBC_SCB_CLK_INTERNAL) */

            /* Enable interrupt source to wakeup device */
            SBC_SetI2CExtClkInterruptMode(enableInt);
        }
        #else
        {
            for(;;) /* Wait for end of transaction intended to slave */
            {
                if(0u == (SBC_GetI2CExtClkInterruptSource() & SBC_INTR_I2C_EC_WAKE_UP))
                {
                    enableInt = CyEnterCriticalSection();

                    if(0u == (SBC_GetI2CExtClkInterruptSource() & SBC_INTR_I2C_EC_WAKE_UP))
                    {
                        /* Attempts to set NACK command before disable block */
                        SBC_I2C_SLAVE_GENERATE_NACK;

                        if(0u == (SBC_GetI2CExtClkInterruptSource() & SBC_INTR_I2C_EC_WAKE_UP))
                        {
                            /* NACK command was set before. It is safe to disable block */
                            SBC_CTRL_REG &= (uint32) ~SBC_CTRL_ENABLED;
                            SBC_DisableInt();

                            CyExitCriticalSection(enableInt);
                            break;
                        }
                        else
                        {
                            /* Clear NACK command to prevent data NACK */
                            SBC_I2C_SLAVE_CLEAR_NACK;
                        }
                    }

                    CyExitCriticalSection(enableInt);
                }
            }

            /* Disable all active mode interrupt sources */
            SBC_SetTxInterruptMode(SBC_NO_INTR_SOURCES);
            SBC_SetSlaveInterruptMode(SBC_NO_INTR_SOURCES);
            SBC_ClearPendingInt();
            SBC_EnableInt();

            /* Enable wakeup interrupt on address match */
            SBC_SetI2CExtClkInterruptMode(SBC_INTR_I2C_EC_WAKE_UP);

            enableInt = CyEnterCriticalSection();

            SBC_CTRL_REG |= (uint32) SBC_CTRL_ENABLED;
            SBC_I2C_SLAVE_GENERATE_NACK;

            CyExitCriticalSection(enableInt);
        }
        #endif
    #endif /* (SBC_CY_SCBIP_V0) */
    }


    /*******************************************************************************
    * Function Name: SBC_EzI2CRestoreConfig
    ********************************************************************************
    *
    * Summary:
    *  Clock stretching is  enabled: Disables the INTR_I2C_EC.WAKE_UP interrupt
    *  source.
    *  Clock stretching is disabled: Reconfigures the EZI2C component from
    *  Deep Sleep (wake up on the address match) to active operation: disables
    *  the INTR_I2C_EC.WAKE_UP interrupt source and restores the INTR_S
    *  interrupt sources to operate in the active mode.
    *  The block is disabled before reconfiguration and enabled when
    *  it is completed.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SBC_EzI2CRestoreConfig(void)
    {
    #if(SBC_CY_SCBIP_V0)

        #if(SBC_EZI2C_SCL_STRETCH_ENABLE)
        {
            /* Disable wakeup interrupt on address match */
            SBC_SetI2CExtClkInterruptMode(SBC_NO_INTR_SOURCES);
        }
        #endif

    #else

        #if(SBC_EZI2C_SCL_STRETCH_ENABLE)
        {
            /* Disable wakeup interrupt source on address match */
            SBC_SetI2CExtClkInterruptMode(SBC_NO_INTR_SOURCES);

        #if (SBC_SCB_CLK_INTERNAL)
            /* Enable clock to internal address match logic. Ticket ID #187931 */
            SBC_SCBCLK_Start();
        #endif /* (SBC_SCB_CLK_INTERNAL) */
        }
        #else
        {
            /* NACK will be driven on the bus by wakeup or NACK command.
            * It is safe to disable block to restore active mode configuration.
            */
            SBC_CTRL_REG &= (uint32) ~SBC_CTRL_ENABLED;

            /* Restore active mode interrupt sources */
            SBC_SetI2CExtClkInterruptMode(SBC_NO_INTR_SOURCES);
            SBC_SetSlaveInterruptMode(SBC_EZI2C_INTR_SLAVE_MASK |
                                                   SBC_INTR_SLAVE_I2C_START);
            SBC_ClearPendingInt();

            SBC_CTRL_REG |= (uint32) SBC_CTRL_ENABLED;
        }
        #endif

    #endif /* (SBC_CY_SCBIP_V0) */
    }
#endif /* (SBC_EZI2C_WAKE_ENABLE_CONST) */


/* [] END OF FILE */
