/*******************************************************************************
* File Name: SBC_PM.c
* Version 3.0
*
* Description:
*  This file provides the source code to the Power Management support for
*  the SCB Component.
*
* Note:
*
********************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SBC.h"
#include "SBC_PVT.h"

#if(SBC_SCB_MODE_I2C_INC)
    #include "SBC_I2C_PVT.h"
#endif /* (SBC_SCB_MODE_I2C_INC) */

#if(SBC_SCB_MODE_EZI2C_INC)
    #include "SBC_EZI2C_PVT.h"
#endif /* (SBC_SCB_MODE_EZI2C_INC) */

#if(SBC_SCB_MODE_SPI_INC || SBC_SCB_MODE_UART_INC)
    #include "SBC_SPI_UART_PVT.h"
#endif /* (SBC_SCB_MODE_SPI_INC || SBC_SCB_MODE_UART_INC) */


/***************************************
*   Backup Structure declaration
***************************************/

#if(SBC_SCB_MODE_UNCONFIG_CONST_CFG || \
   (SBC_SCB_MODE_I2C_CONST_CFG   && (!SBC_I2C_WAKE_ENABLE_CONST))   || \
   (SBC_SCB_MODE_EZI2C_CONST_CFG && (!SBC_EZI2C_WAKE_ENABLE_CONST)) || \
   (SBC_SCB_MODE_SPI_CONST_CFG   && (!SBC_SPI_WAKE_ENABLE_CONST))   || \
   (SBC_SCB_MODE_UART_CONST_CFG  && (!SBC_UART_WAKE_ENABLE_CONST)))

    SBC_BACKUP_STRUCT SBC_backup =
    {
        0u, /* enableState */
    };
#endif


/*******************************************************************************
* Function Name: SBC_Sleep
********************************************************************************
*
* Summary:
*  Prepares the component to enter Deep Sleep.
*  The "Enable wakeup from Sleep Mode" selection has an influence on
*  this function implementation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SBC_Sleep(void)
{
#if(SBC_SCB_MODE_UNCONFIG_CONST_CFG)

    if(SBC_SCB_WAKE_ENABLE_CHECK)
    {
        if(SBC_SCB_MODE_I2C_RUNTM_CFG)
        {
            SBC_I2CSaveConfig();
        }
        else if(SBC_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            SBC_EzI2CSaveConfig();
        }
    #if(!SBC_CY_SCBIP_V1)
        else if(SBC_SCB_MODE_SPI_RUNTM_CFG)
        {
            SBC_SpiSaveConfig();
        }
        else if(SBC_SCB_MODE_UART_RUNTM_CFG)
        {
            SBC_UartSaveConfig();
        }
    #endif /* (!SBC_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        SBC_backup.enableState = (uint8) SBC_GET_CTRL_ENABLED;

        if(0u != SBC_backup.enableState)
        {
            SBC_Stop();
        }
    }

#else

    #if (SBC_SCB_MODE_I2C_CONST_CFG && SBC_I2C_WAKE_ENABLE_CONST)
        SBC_I2CSaveConfig();

    #elif (SBC_SCB_MODE_EZI2C_CONST_CFG && SBC_EZI2C_WAKE_ENABLE_CONST)
        SBC_EzI2CSaveConfig();

    #elif (SBC_SCB_MODE_SPI_CONST_CFG && SBC_SPI_WAKE_ENABLE_CONST)
        SBC_SpiSaveConfig();

    #elif (SBC_SCB_MODE_UART_CONST_CFG && SBC_UART_WAKE_ENABLE_CONST)
        SBC_UartSaveConfig();

    #else

        SBC_backup.enableState = (uint8) SBC_GET_CTRL_ENABLED;

        if(0u != SBC_backup.enableState)
        {
            SBC_Stop();
        }

    #endif /* defined (SBC_SCB_MODE_I2C_CONST_CFG) && (SBC_I2C_WAKE_ENABLE_CONST) */

#endif /* (SBC_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SBC_Wakeup
********************************************************************************
*
* Summary:
*  Prepares the component for the Active mode operation after exiting
*  Deep Sleep. The "Enable wakeup from Sleep Mode" option has an influence
*  on this function implementation.
*  This function should not be called after exiting Sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SBC_Wakeup(void)
{
#if(SBC_SCB_MODE_UNCONFIG_CONST_CFG)

    if(SBC_SCB_WAKE_ENABLE_CHECK)
    {
        if(SBC_SCB_MODE_I2C_RUNTM_CFG)
        {
            SBC_I2CRestoreConfig();
        }
        else if(SBC_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            SBC_EzI2CRestoreConfig();
        }
    #if(!SBC_CY_SCBIP_V1)
        else if(SBC_SCB_MODE_SPI_RUNTM_CFG)
        {
            SBC_SpiRestoreConfig();
        }
        else if(SBC_SCB_MODE_UART_RUNTM_CFG)
        {
            SBC_UartRestoreConfig();
        }
    #endif /* (!SBC_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        if(0u != SBC_backup.enableState)
        {
            SBC_Enable();
        }
    }

#else

    #if (SBC_SCB_MODE_I2C_CONST_CFG  && SBC_I2C_WAKE_ENABLE_CONST)
        SBC_I2CRestoreConfig();

    #elif (SBC_SCB_MODE_EZI2C_CONST_CFG && SBC_EZI2C_WAKE_ENABLE_CONST)
        SBC_EzI2CRestoreConfig();

    #elif (SBC_SCB_MODE_SPI_CONST_CFG && SBC_SPI_WAKE_ENABLE_CONST)
        SBC_SpiRestoreConfig();

    #elif (SBC_SCB_MODE_UART_CONST_CFG && SBC_UART_WAKE_ENABLE_CONST)
        SBC_UartRestoreConfig();

    #else

        if(0u != SBC_backup.enableState)
        {
            SBC_Enable();
        }

    #endif /* (SBC_I2C_WAKE_ENABLE_CONST) */

#endif /* (SBC_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/* [] END OF FILE */
